@@ -6,10 +6,14 @@
 
 set -eu
 
-cargo doc --workspace --document-private-items --no-deps
+# turn warnings into errors
+export RUSTDOCFLAGS='-D warnings'
+
+cargo doc --document-private-items --no-deps --package librsvg
 mkdir -p public/internals
 mv target/doc/* public/internals
 
-cargo doc --no-deps --package librsvg --package 'librsvg-rebind*'
+# cargo doc --no-deps --package librsvg --package 'librsvg-rebind*'
+cargo doc --no-deps --package librsvg
 mkdir -p public/doc
 cp -r target/doc/* public/doc
