@@ -78,16 +78,18 @@ use cssparser::{
     DeclarationParser, Parser, ParserInput, ParserState, QualifiedRuleParser, RuleBodyItemParser,
     RuleBodyParser, SourceLocation, StyleSheetParser, ToCss,
 };
-use data_url::mime::Mime;
 use language_tags::LanguageTag;
-use markup5ever::{self, namespace_url, ns, Namespace, QualName};
+use markup5ever::{self, ns, Namespace, QualName};
+use precomputed_hash::PrecomputedHash;
 use selectors::attr::{AttrSelectorOperation, CaseSensitivity, NamespaceConstraint};
+use selectors::bloom::BloomFilter;
+use selectors::context::SelectorCaches;
 use selectors::matching::{
-    ElementSelectorFlags, IgnoreNthChildForInvalidation, MatchingContext, MatchingMode,
+    ElementSelectorFlags, MatchingContext, MatchingForInvalidation, MatchingMode,
     NeedsSelectorFlags, QuirksMode,
 };
 use selectors::parser::ParseRelative;
-use selectors::{NthIndexCache, OpaqueElement, SelectorImpl, SelectorList};
+use selectors::{OpaqueElement, SelectorImpl, SelectorList};
 use std::cmp::Ordering;
 use std::fmt;
 use std::str;
@@ -95,7 +97,7 @@ use std::str::FromStr;
 
 use crate::element::Element;
 use crate::error::*;
-use crate::io::{self, BinaryData};
+use crate::io;
 use crate::node::{Node, NodeBorrow, NodeCascade};
 use crate::properties::{parse_value, ComputedValues, ParseAs, ParsedProperty};
 use crate::rsvg_log;
@@ -137,6 +139,7 @@ impl<'i> DeclarationParser<'i> for DeclParser {
         &mut self,
         name: CowRcStr<'i>,
         input: &mut Parser<'i, 't>,
+        _declaration_start: &ParserState,
     ) -> Result<RuleBodyItem, cssparser::ParseError<'i, Self::Error>> {
         let prop_name = QualName::new(None, ns!(), markup5ever::LocalName::from(name.as_ref()));
         let property = parse_value(&prop_name, input, ParseAs::Property)?;
@@ -256,6 +259,7 @@ impl<'i> selectors::Parser<'i> for RuleParser {
         &self,
         name: CowRcStr<'i>,
         arguments: &mut Parser<'i, '_>,
+        _after_part: bool,
     ) -> Result<NonTSPseudoClass, cssparser::ParseError<'i, Self::Error>> {
         match &*name {
             "lang" => {
@@ -492,6 +496,12 @@ impl ToCss for Identifier {
     }
 }
 
+impl PrecomputedHash for Identifier {
+    fn precomputed_hash(&self) -> u32 {
+        self.0.precomputed_hash()
+    }
+}
+
 /// Wrapper for local names.
 ///
 /// Used to implement `ToCss` on the `LocalName` foreign type.
@@ -513,6 +523,12 @@ impl ToCss for LocalName {
     }
 }
 
+impl PrecomputedHash for LocalName {
+    fn precomputed_hash(&self) -> u32 {
+        self.0.precomputed_hash()
+    }
+}
+
 /// Wrapper for namespace prefixes.
 ///
 /// Used to implement `ToCss` on the `markup5ever::Prefix` foreign type.
@@ -757,6 +773,10 @@ where {
             .unwrap_or(false)
     }
 
+    fn has_custom_state(&self, _name: &<Self::Impl as SelectorImpl>::Identifier) -> bool {
+        false
+    }
+
     fn imported_part(&self, _name: &Identifier) -> Option<Identifier> {
         // unsupported
         None
@@ -787,6 +807,10 @@ where {
         self.0.parent().is_none()
     }
 
+    fn add_element_unique_hashes(&self, _filter: &mut BloomFilter) -> bool {
+        false
+    }
+
     /// Returns the first child element of this element.
     fn first_element_child(&self) -> Option<Self> {
         self.0
@@ -941,20 +965,7 @@ impl Stylesheet {
         io::acquire_data(aurl, None)
             .map_err(LoadingError::from)
             .and_then(|data| {
-                let BinaryData {
-                    data: bytes,
-                    mime_type,
-                } = data;
-
-                if is_text_css(&mime_type) {
-                    Ok(bytes)
-                } else {
-                    rsvg_log!(session, "\"{}\" is not of type text/css; ignoring", aurl);
-                    Err(LoadingError::BadCss)
-                }
-            })
-            .and_then(|bytes| {
-                String::from_utf8(bytes).map_err(|_| {
+                String::from_utf8(data.data).map_err(|_| {
                     rsvg_log!(
                         session,
                         "\"{}\" does not contain valid UTF-8 CSS data; ignoring",
@@ -977,7 +988,7 @@ impl Stylesheet {
         acc: &mut Vec<Match<'a>>,
     ) {
         for rule in &self.qualified_rules {
-            for selector in &rule.selectors.0 {
+            for selector in rule.selectors.slice() {
                 // This magic call is stolen from selectors::matching::matches_selector_list()
                 let matches = selectors::matching::matches_selector(
                     selector,
@@ -1001,10 +1012,6 @@ impl Stylesheet {
     }
 }
 
-fn is_text_css(mime_type: &Mime) -> bool {
-    mime_type.type_ == "text" && mime_type.subtype == "css"
-}
-
 /// Runs the CSS cascade on the specified tree from all the stylesheets
 pub fn cascade(
     root: &mut Node,
@@ -1022,15 +1029,15 @@ pub fn cascade(
         let parent = node.parent().clone();
         node.borrow_element_mut().inherit_xml_lang(parent);
 
-        let mut cache = NthIndexCache::default();
+        let mut caches = SelectorCaches::default();
         let mut match_ctx = MatchingContext::new(
             MatchingMode::Normal,
             // FIXME: how the fuck does one set up a bloom filter here?
             None,
-            &mut cache,
+            &mut caches,
             QuirksMode::NoQuirks,
             NeedsSelectorFlags::No,
-            IgnoreNthChildForInvalidation::No,
+            MatchingForInvalidation::No,
         );
 
         for s in ua_stylesheets
