@@ -1,6 +1,6 @@
 //! SVG Elements.
 
-use markup5ever::{expanded_name, local_name, namespace_url, ns, QualName};
+use markup5ever::{expanded_name, local_name, ns, QualName};
 use std::collections::{HashMap, HashSet};
 use std::fmt;
 use std::sync::OnceLock;
@@ -48,6 +48,8 @@ use crate::text::{TRef, TSpan, Text};
 use crate::text2::Text2;
 use crate::xml::Attributes;
 
+pub type DrawResult = Result<Box<BoundingBox>, Box<InternalRenderingError>>;
+
 pub trait ElementTrait {
     /// Sets per-element attributes.
     ///
@@ -66,12 +68,12 @@ pub trait ElementTrait {
         _node: &Node,
         _acquired_nodes: &mut AcquiredNodes<'_>,
         _cascaded: &CascadedValues<'_>,
-        _viewport: &Viewport,
-        draw_ctx: &mut DrawingCtx,
+        viewport: &Viewport,
+        _draw_ctx: &mut DrawingCtx,
         _clipping: bool,
-    ) -> Result<BoundingBox, InternalRenderingError> {
+    ) -> DrawResult {
         // by default elements don't draw themselves
-        Ok(draw_ctx.empty_bbox())
+        Ok(viewport.empty_bbox())
     }
 
     /// Create a layout object for the current element.
@@ -86,7 +88,7 @@ pub trait ElementTrait {
         _viewport: &Viewport,
         _draw_ctx: &mut DrawingCtx,
         _clipping: bool,
-    ) -> Result<Option<Layer>, InternalRenderingError> {
+    ) -> Result<Option<Layer>, Box<InternalRenderingError>> {
         Ok(None)
     }
 }
@@ -495,13 +497,13 @@ impl Element {
         viewport: &Viewport,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
-    ) -> Result<BoundingBox, InternalRenderingError> {
+    ) -> DrawResult {
         let values = cascaded.get();
         if values.is_displayed() {
             self.element_data
                 .draw(node, acquired_nodes, cascaded, viewport, draw_ctx, clipping)
         } else {
-            Ok(draw_ctx.empty_bbox())
+            Ok(viewport.empty_bbox())
         }
     }
 
@@ -514,7 +516,7 @@ impl Element {
         viewport: &Viewport,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
-    ) -> Result<Option<Layer>, InternalRenderingError> {
+    ) -> Result<Option<Layer>, Box<InternalRenderingError>> {
         let values = cascaded.get();
         if values.is_displayed() {
             self.element_data
@@ -536,7 +538,7 @@ impl ElementData {
         viewport: &Viewport,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
-    ) -> Result<BoundingBox, InternalRenderingError> {
+    ) -> DrawResult {
         use ElementData::*;
 
         let data: &dyn ElementTrait = match self {
@@ -609,7 +611,7 @@ impl ElementData {
         viewport: &Viewport,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
-    ) -> Result<Option<Layer>, InternalRenderingError> {
+    ) -> Result<Option<Layer>, Box<InternalRenderingError>> {
         use ElementData::*;
 
         let data: &dyn ElementTrait = match self {
