@@ -188,6 +188,20 @@ impl From<cairo::Error> for InternalRenderingError {
     }
 }
 
+macro_rules! box_error {
+    ($from_ty:ty) => {
+        impl From<$from_ty> for Box<InternalRenderingError> {
+            fn from(e: $from_ty) -> Box<InternalRenderingError> {
+                Box::new(e.into())
+            }
+        }
+    };
+}
+
+box_error!(DefsLookupErrorKind);
+box_error!(InvalidTransform);
+box_error!(cairo::Error);
+
 /// Indicates that a transform is not invertible.
 ///
 /// This generally represents an error from [`crate::transform::ValidTransform::try_from`], which is what we use
@@ -284,41 +298,49 @@ impl<O, E: Into<ValueErrorKind>> AttributeResultExt<O> for Result<O, E> {
     }
 }
 
+/// Convert a short-lived ParseError into a long-lived ElementError
+///
+/// We extract this as a function, instead of putting it directly in the `map_err` invocation
+/// below in `impl<'i, O> AttributeResultExt<O> for Result<O, ParseError<'i>>`, because
+/// putting it there as a closure generates too many duplicated copies of the code.  The generic
+/// parameter `O` in that `impl` is for the result `Ok()` value, not for the `Err`, after all.
+fn parse_error_to_element_error<'i>(e: ParseError<'i>, attr: QualName) -> ElementError {
+    // FIXME: eventually, here we'll want to preserve the location information
+
+    let ParseError {
+        kind,
+        location: _location,
+    } = e;
+
+    match kind {
+        ParseErrorKind::Basic(BasicParseErrorKind::UnexpectedToken(tok)) => {
+            let mut s = String::from("unexpected token '");
+            tok.to_css(&mut s).unwrap(); // FIXME: what do we do with a fmt::Error?
+            s.push('\'');
+
+            ElementError {
+                attr,
+                err: ValueErrorKind::Parse(s),
+            }
+        }
+
+        ParseErrorKind::Basic(BasicParseErrorKind::EndOfInput) => ElementError {
+            attr,
+            err: ValueErrorKind::parse_error("unexpected end of input"),
+        },
+
+        ParseErrorKind::Basic(_) => {
+            unreachable!("attribute parsers should not return errors for CSS rules")
+        }
+
+        ParseErrorKind::Custom(err) => ElementError { attr, err },
+    }
+}
+
 /// Turns a short-lived `ParseError` into a long-lived `ElementError`
 impl<'i, O> AttributeResultExt<O> for Result<O, ParseError<'i>> {
     fn attribute(self, attr: QualName) -> Result<O, ElementError> {
-        self.map_err(|e| {
-            // FIXME: eventually, here we'll want to preserve the location information
-
-            let ParseError {
-                kind,
-                location: _location,
-            } = e;
-
-            match kind {
-                ParseErrorKind::Basic(BasicParseErrorKind::UnexpectedToken(tok)) => {
-                    let mut s = String::from("unexpected token '");
-                    tok.to_css(&mut s).unwrap(); // FIXME: what do we do with a fmt::Error?
-                    s.push('\'');
-
-                    ElementError {
-                        attr,
-                        err: ValueErrorKind::Parse(s),
-                    }
-                }
-
-                ParseErrorKind::Basic(BasicParseErrorKind::EndOfInput) => ElementError {
-                    attr,
-                    err: ValueErrorKind::parse_error("unexpected end of input"),
-                },
-
-                ParseErrorKind::Basic(_) => {
-                    unreachable!("attribute parsers should not return errors for CSS rules")
-                }
-
-                ParseErrorKind::Custom(err) => ElementError { attr, err },
-            }
-        })
+        self.map_err(|e| parse_error_to_element_error(e, attr))
     }
 }
 
@@ -347,8 +369,14 @@ pub enum AllowedUrlError {
     /// URLs may not have fragment identifiers at this stage
     NoFragmentIdentifierAllowed,
 
-    /// Error when obtaining the file path or the base file path
-    InvalidPath,
+    /// `file:` URLs may not have a hostname
+    NoHostAllowed,
+
+    /// Error when obtaining the file path that corresponds to the URL
+    InvalidPathInUrl,
+
+    /// Error when obtaining the file path that corresponds to the base URL
+    InvalidPathInBaseUrl,
 
     /// The base file cannot be the root of the file system
     BaseIsRoot,
@@ -368,7 +396,9 @@ impl fmt::Display for AllowedUrlError {
             NotSiblingOrChildOfBaseFile => write!(f, "not sibling or child of base file"),
             NoQueriesAllowed => write!(f, "no queries allowed"),
             NoFragmentIdentifierAllowed => write!(f, "no fragment identifier allowed"),
-            InvalidPath => write!(f, "invalid path"),
+            NoHostAllowed => write!(f, "no hostnames allowed"),
+            InvalidPathInUrl => write!(f, "invalid path in file URL"),
+            InvalidPathInBaseUrl => write!(f, "invalid path in base URL"),
             BaseIsRoot => write!(f, "base is root"),
             CanonicalizationError => write!(f, "canonicalization error"),
         }
@@ -480,6 +510,13 @@ pub enum ImplementationLimit {
     /// number of attributes that the SVG standard ascribes meaning to are
     /// lower than this limit.
     TooManyAttributes,
+
+    /// Document exceeded the maximum nesting level while rendering.
+    ///
+    /// Rendering is a recursive process, and there is a limit of how deep layers can
+    /// nest.  This is to avoid malicious SVGs which try to have layers that are nested
+    /// extremely deep, as this could cause stack exhaustion.
+    MaximumLayerNestingDepthExceeded,
 }
 
 impl error::Error for LoadingError {}
@@ -536,6 +573,12 @@ impl fmt::Display for ImplementationLimit {
                 "cannot load more than {} XML attributes",
                 limits::MAX_LOADED_ATTRIBUTES
             ),
+
+            ImplementationLimit::MaximumLayerNestingDepthExceeded => write!(
+                f,
+                "maximum depth of {} nested layers has been exceeded",
+                limits::MAX_LAYER_NESTING_DEPTH,
+            ),
         }
     }
 }
