@@ -1,25 +1,22 @@
 //! Lighting filters and light nodes.
 
-use cssparser::{Color, RGBA};
 use float_cmp::approx_eq;
-use markup5ever::{expanded_name, local_name, namespace_url, ns};
+use markup5ever::{expanded_name, local_name, ns};
 use nalgebra::{Vector2, Vector3};
 use num_traits::identities::Zero;
 use rayon::prelude::*;
 use std::cmp::max;
 
-use crate::color::color_to_rgba;
+use crate::color::{color_to_rgba, resolve_color, Color, RGBA};
 use crate::document::AcquiredNodes;
-use crate::drawing_ctx::DrawingCtx;
 use crate::element::{set_attribute, ElementData, ElementTrait};
 use crate::filters::{
     bounds::BoundsBuilder,
     context::{FilterContext, FilterOutput},
-    FilterEffect, FilterError, FilterResolveError, Input, Primitive, PrimitiveParams,
-    ResolvedPrimitive,
+    FilterEffect, FilterError, FilterResolveError, Input, InputRequirements, Primitive,
+    PrimitiveParams, ResolvedPrimitive,
 };
 use crate::node::{CascadedValues, Node, NodeBorrow};
-use crate::paint_server::resolve_color;
 use crate::parsers::{NonNegative, ParseValue};
 use crate::properties::ColorInterpolationFilters;
 use crate::rect::IRect;
@@ -150,7 +147,7 @@ fn color_and_vector(
     x: f64,
     y: f64,
     z: f64,
-) -> (cssparser::RGBA, Vector3<f64>) {
+) -> (RGBA, Vector3<f64>) {
     let vector = match *source {
         LightSource::Distant { azimuth, elevation } => {
             let azimuth = azimuth.to_radians();
@@ -175,7 +172,7 @@ fn color_and_vector(
             limiting_cone_angle,
             ..
         } => {
-            let transparent_color = cssparser::RGBA::new(Some(0), Some(0), Some(0), Some(0.0));
+            let transparent_color = RGBA::new(0, 0, 0, 0.0);
             let minus_l_dot_s = -vector.dot(&direction);
             match limiting_cone_angle {
                 _ if minus_l_dot_s <= 0.0 => transparent_color,
@@ -184,11 +181,11 @@ fn color_and_vector(
                     let factor = minus_l_dot_s.powf(specular_exponent);
                     let compute = |x| (clamp(f64::from(x) * factor, 0.0, 255.0) + 0.5) as u8;
 
-                    cssparser::RGBA {
-                        red: Some(compute(lighting_color.red.unwrap_or(0))),
-                        green: Some(compute(lighting_color.green.unwrap_or(0))),
-                        blue: Some(compute(lighting_color.blue.unwrap_or(0))),
-                        alpha: Some(1.0),
+                    RGBA {
+                        red: compute(lighting_color.red),
+                        green: compute(lighting_color.green),
+                        blue: compute(lighting_color.blue),
+                        alpha: 1.0,
                     }
                 }
             }
@@ -464,15 +461,9 @@ macro_rules! impl_lighting_filter {
                 &self,
                 bounds_builder: BoundsBuilder,
                 ctx: &FilterContext,
-                acquired_nodes: &mut AcquiredNodes<'_>,
-                draw_ctx: &mut DrawingCtx,
             ) -> Result<FilterOutput, FilterError> {
-                let input_1 = ctx.get_input(
-                    acquired_nodes,
-                    draw_ctx,
-                    &self.params.in1,
-                    self.light.color_interpolation_filters,
-                )?;
+                let input_1 =
+                    ctx.get_input(&self.params.in1, self.light.color_interpolation_filters)?;
                 let mut bounds: IRect = bounds_builder
                     .add_input(&input_1)
                     .compute(ctx)
@@ -538,9 +529,9 @@ macro_rules! impl_lighting_filter {
                             let compute =
                                 |x| (clamp(factor * f64::from(x), 0.0, 255.0) + 0.5) as u8;
 
-                            let r = compute(color.red.unwrap_or(0));
-                            let g = compute(color.green.unwrap_or(0));
-                            let b = compute(color.blue.unwrap_or(0));
+                            let r = compute(color.red);
+                            let g = compute(color.green);
+                            let b = compute(color.blue);
                             let a = $alpha_func(r, g, b);
 
                             let output_pixel = Pixel { r, g, b, a };
@@ -673,6 +664,10 @@ macro_rules! impl_lighting_filter {
 
                 Ok(FilterOutput { surface, bounds })
             }
+
+            pub fn get_input_requirements(&self) -> InputRequirements {
+                self.params.in1.get_requirements()
+            }
         }
 
         impl FilterEffect for $lighting_type {
