@@ -1,8 +1,7 @@
 use cssparser::Parser;
-use markup5ever::{expanded_name, local_name, namespace_url, ns};
+use markup5ever::{expanded_name, local_name, ns};
 
 use crate::document::AcquiredNodes;
-use crate::drawing_ctx::DrawingCtx;
 use crate::element::{set_attribute, ElementTrait};
 use crate::error::*;
 use crate::node::{CascadedValues, Node};
@@ -22,12 +21,13 @@ use crate::xml::Attributes;
 use super::bounds::BoundsBuilder;
 use super::context::{FilterContext, FilterOutput};
 use super::{
-    FilterEffect, FilterError, FilterResolveError, Primitive, PrimitiveParams, ResolvedPrimitive,
+    FilterEffect, FilterError, FilterResolveError, InputRequirements, Primitive, PrimitiveParams,
+    ResolvedPrimitive,
 };
 
 /// Limit the `numOctaves` parameter to avoid unbounded CPU consumption.
 ///
-/// https://drafts.fxtf.org/filter-effects/#element-attrdef-feturbulence-numoctaves
+/// <https://drafts.fxtf.org/filter-effects/#element-attrdef-feturbulence-numoctaves>
 const MAX_OCTAVES: i32 = 9;
 
 /// Enumeration of the tile stitching modes.
@@ -355,8 +355,6 @@ impl Turbulence {
         &self,
         bounds_builder: BoundsBuilder,
         ctx: &FilterContext,
-        _acquired_nodes: &mut AcquiredNodes<'_>,
-        _draw_ctx: &mut DrawingCtx,
     ) -> Result<FilterOutput, FilterError> {
         let bounds: IRect = bounds_builder.compute(ctx).clipped.into();
 
@@ -454,6 +452,10 @@ impl Turbulence {
             bounds,
         })
     }
+
+    pub fn get_input_requirements(&self) -> InputRequirements {
+        InputRequirements::default()
+    }
 }
 
 impl FilterEffect for FeTurbulence {
