@@ -160,11 +160,11 @@ make_property!(
     /// override this by specifing a custom CSS stylesheet.
     ///
     /// Most of the time the `color` property is used to call
-    /// [`crate::paint_server::resolve_color`].
+    /// [`crate::color::resolve_color`].
     Color,
-    default: cssparser::Color::Rgba(cssparser::RGBA::new(Some(0), Some(0), Some(0), Some(1.0))),
+    default: crate::color::Color::Rgba(crate::color::RGBA::new(0, 0, 0, 1.0)),
     inherits_automatically: true,
-    newtype_parse: cssparser::Color,
+    newtype_parse: crate::color::Color,
 );
 
 make_property!(
@@ -354,8 +354,8 @@ make_property!(
     ///
     /// SVG2: <https://www.w3.org/TR/SVG2/painting.html#FillProperty>
     Fill,
-    default: PaintServer::SolidColor(cssparser::Color::Rgba(
-        cssparser::RGBA::new(Some(0), Some(0), Some(0), Some(1.0))
+    default: PaintServer::SolidColor(crate::color::Color::Rgba(
+        crate::color::RGBA::new(0, 0, 0, 1.0)
     )),
     inherits_automatically: true,
     newtype_parse: PaintServer,
@@ -430,9 +430,9 @@ make_property!(
     ///
     /// Filter Effects 1: <https://www.w3.org/TR/filter-effects/#FloodColorProperty>
     FloodColor,
-    default: cssparser::Color::Rgba(cssparser::RGBA::new(Some(0), Some(0), Some(0), Some(1.0))),
+    default: crate::color::Color::Rgba(crate::color::RGBA::new(0, 0, 0, 1.0)),
     inherits_automatically: false,
-    newtype_parse: cssparser::Color,
+    newtype_parse: crate::color::Color,
 );
 
 make_property!(
@@ -641,9 +641,9 @@ make_property!(
     ///
     /// Filter Effects 1: <https://www.w3.org/TR/filter-effects/#LightingColorProperty>
     LightingColor,
-    default: cssparser::Color::Rgba(cssparser::RGBA::new(Some(255), Some(255), Some(255), Some(1.0))),
+    default: crate::color::Color::Rgba(crate::color::RGBA::new(255, 255, 255, 1.0)),
     inherits_automatically: false,
-    newtype_parse: cssparser::Color,
+    newtype_parse: crate::color::Color,
 );
 
 make_property!(
@@ -923,9 +923,9 @@ make_property!(
     ///
     /// SVG2: <https://www.w3.org/TR/SVG2/pservers.html#StopColorProperty>
     StopColor,
-    default: cssparser::Color::Rgba(cssparser::RGBA::new(Some(0), Some(0), Some(0), Some(1.0))),
+    default: crate::color::Color::Rgba(crate::color::RGBA::new(0, 0, 0, 1.0)),
     inherits_automatically: false,
-    newtype_parse: cssparser::Color,
+    newtype_parse: crate::color::Color,
 );
 
 make_property!(
@@ -1397,7 +1397,7 @@ make_property!(
 make_property!(
     /// `whitespace` properties
     ///
-    /// https://www.w3.org/TR/css-text-3/#white-space-property
+    /// <https://www.w3.org/TR/css-text-3/#white-space-property>
     WhiteSpace,
 
     default: Normal,
