@@ -1,16 +1,15 @@
 //! Text elements: `text`, `tspan`, `tref`.
 
-use markup5ever::{expanded_name, local_name, namespace_url, ns, QualName};
+use markup5ever::{expanded_name, local_name, ns, QualName};
 use pango::prelude::FontExt;
 use pango::IsAttribute;
 use std::cell::RefCell;
 use std::convert::TryFrom;
 use std::rc::Rc;
 
-use crate::bbox::BoundingBox;
 use crate::document::{AcquiredNodes, NodeId};
 use crate::drawing_ctx::{create_pango_context, DrawingCtx, FontOptions, Viewport};
-use crate::element::{set_attribute, ElementData, ElementTrait};
+use crate::element::{set_attribute, DrawResult, ElementData, ElementTrait};
 use crate::error::*;
 use crate::layout::{self, FontProperties, Layer, LayerKind, StackingContext, Stroke, TextSpan};
 use crate::length::*;
@@ -25,7 +24,6 @@ use crate::rect::Rect;
 use crate::rsvg_log;
 use crate::session::Session;
 use crate::space::{xml_space_normalize, NormalizeDefault, XmlSpaceNormalize};
-use crate::transform::{Transform, ValidTransform};
 use crate::xml::Attributes;
 
 /// The state of a text layout operation.
@@ -33,9 +31,6 @@ struct LayoutContext {
     /// `writing-mode` property from the `<text>` element.
     writing_mode: WritingMode,
 
-    /// Current transform in the DrawingCtx.
-    transform: ValidTransform,
-
     /// Font options from the DrawingCtx.
     font_options: FontOptions,
 
@@ -116,7 +111,7 @@ struct PositionedSpan {
 struct LayoutSpan {
     layout: pango::Layout,
     gravity: pango::Gravity,
-    bbox: Option<BoundingBox>,
+    extents: Option<Rect>,
     is_visible: bool,
     x: f64,
     y: f64,
@@ -295,6 +290,12 @@ fn compute_baseline_offset(
     loop {
         if let Some(layout_run) = layout_iter.run_readonly() {
             let item = layout_run.item();
+            unsafe {
+                let analysis = (*item.as_ptr()).analysis;
+                if analysis.font.is_null() {
+                    break;
+                }
+            }
             let font = item.analysis().font();
 
             let metrics = font.metrics(None);
@@ -468,9 +469,8 @@ fn compute_text_box(
     layout: &pango::Layout,
     x: f64,
     y: f64,
-    transform: Transform,
     gravity: pango::Gravity,
-) -> Option<BoundingBox> {
+) -> Option<Rect> {
     #![allow(clippy::many_single_char_names)]
 
     let (ink, _) = layout.extents();
@@ -500,13 +500,7 @@ fn compute_text_box(
         )
     };
 
-    let r = Rect::new(x, y, x + w, y + h);
-    let bbox = BoundingBox::new()
-        .with_transform(transform)
-        .with_rect(r)
-        .with_ink_rect(r);
-
-    Some(bbox)
+    Some(Rect::new(x, y, x + w, y + h))
 }
 
 impl PositionedSpan {
@@ -525,7 +519,7 @@ impl PositionedSpan {
 
         let gravity = layout.context().gravity();
 
-        let bbox = compute_text_box(&layout, x, y, *layout_context.transform, gravity);
+        let extents = compute_text_box(&layout, x, y, gravity);
 
         let stroke_paint = self.values.stroke().0.resolve(
             acquired_nodes,
@@ -551,7 +545,7 @@ impl PositionedSpan {
         LayoutSpan {
             layout,
             gravity,
-            bbox,
+            extents,
             is_visible,
             x,
             y,
@@ -860,7 +854,7 @@ impl ElementTrait for Text {
         viewport: &Viewport,
         draw_ctx: &mut DrawingCtx,
         _clipping: bool,
-    ) -> Result<Option<Layer>, InternalRenderingError> {
+    ) -> Result<Option<Layer>, Box<InternalRenderingError>> {
         let values = cascaded.get();
         let params = NormalizeParams::new(values, viewport);
 
@@ -878,13 +872,10 @@ impl ElementTrait for Text {
         );
 
         let layout_text = {
-            let transform = ValidTransform::try_from(Transform::identity()).unwrap();
-
             let layout_context = LayoutContext {
                 writing_mode: values.writing_mode(),
-                transform,
                 font_options: draw_ctx.get_font_options(),
-                viewport: viewport.clone(),
+                viewport: *viewport,
                 session: session.clone(),
             };
 
@@ -919,27 +910,28 @@ impl ElementTrait for Text {
                 }
             }
 
-            let empty_bbox = BoundingBox::new().with_transform(*transform);
-
-            let text_bbox = layout_spans.iter().fold(empty_bbox, |mut bbox, span| {
-                if let Some(ref span_bbox) = span.bbox {
-                    bbox.insert(span_bbox);
-                }
-
-                bbox
-            });
+            let text_extents: Option<Rect> = layout_spans
+                .iter()
+                .map(|span| span.extents)
+                .reduce(|a, b| match (a, b) {
+                    (None, None) => None,
+                    (None, Some(b)) => Some(b),
+                    (Some(a), None) => Some(a),
+                    (Some(a), Some(b)) => Some(a.union(&b)),
+                })
+                .flatten();
 
             let mut text_spans = Vec::new();
             for span in layout_spans {
                 let normalize_values = NormalizeValues::new(&span.values);
 
                 let stroke_paint = span.stroke_paint.to_user_space(
-                    &text_bbox.rect,
+                    &text_extents,
                     &layout_context.viewport,
                     &normalize_values,
                 );
                 let fill_paint = span.fill_paint.to_user_space(
-                    &text_bbox.rect,
+                    &text_extents,
                     &layout_context.viewport,
                     &normalize_values,
                 );
@@ -947,7 +939,7 @@ impl ElementTrait for Text {
                 let text_span = TextSpan {
                     layout: span.layout,
                     gravity: span.gravity,
-                    bbox: span.bbox,
+                    extents: span.extents,
                     is_visible: span.is_visible,
                     x: span.x,
                     y: span.y,
@@ -962,7 +954,10 @@ impl ElementTrait for Text {
                 text_spans.push(text_span);
             }
 
-            layout::Text { spans: text_spans }
+            layout::Text {
+                spans: text_spans,
+                extents: text_extents,
+            }
         };
 
         Ok(Some(Layer {
@@ -979,7 +974,7 @@ impl ElementTrait for Text {
         viewport: &Viewport,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
-    ) -> Result<BoundingBox, InternalRenderingError> {
+    ) -> DrawResult {
         self.layout(node, acquired_nodes, cascaded, viewport, draw_ctx, clipping)
             .and_then(|layer| {
                 draw_ctx.draw_layer(layer.as_ref().unwrap(), acquired_nodes, clipping, viewport)
@@ -1208,7 +1203,7 @@ impl From<WritingMode> for pango::Gravity {
 /// Constants with Unicode's directional formatting characters
 ///
 /// <https://unicode.org/reports/tr9/#Directional_Formatting_Characters>
-mod directional_formatting_characters {
+pub mod directional_formatting_characters {
     /// Left-to-Right Embedding
     ///
     /// Treat the following text as embedded left-to-right.
@@ -1262,9 +1257,9 @@ mod directional_formatting_characters {
 /// another.  This struct contains slices with the control characters that must be
 /// inserted into the text stream at the span's limits so that the bidi/shaping engine
 /// will know what to do.
-struct BidiControl {
-    start: &'static [char],
-    end: &'static [char],
+pub struct BidiControl {
+    pub start: &'static [char],
+    pub end: &'static [char],
 }
 
 impl BidiControl {
@@ -1273,7 +1268,7 @@ impl BidiControl {
     /// See the table titled "Bidi control codes injected..." in
     /// <https://www.w3.org/TR/css-writing-modes-3/#unicode-bidi>
     #[rustfmt::skip]
-    fn from_unicode_bidi_and_direction(unicode_bidi: UnicodeBidi, direction: Direction) -> BidiControl {
+    pub fn from_unicode_bidi_and_direction(unicode_bidi: UnicodeBidi, direction: Direction) -> BidiControl {
         use UnicodeBidi::*;
         use Direction::*;
         use directional_formatting_characters::*;
@@ -1320,8 +1315,7 @@ fn create_pango_layout(
     props: &FontProperties,
     text: &str,
 ) -> Option<pango::Layout> {
-    let pango_context =
-        create_pango_context(&layout_context.font_options, &layout_context.transform);
+    let pango_context = create_pango_context(&layout_context.font_options);
 
     if let XmlLang(Some(ref lang)) = props.xml_lang {
         pango_context.set_language(Some(&pango::Language::from_string(lang.as_str())));
